# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.

# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.

__copyright__ = '2013 Canonical Ltd - ' + \
                'Martin Mrazik <martin.mrazik@canonical.com>'

import logging
import os
import re
from launchpadlib.launchpad import Launchpad
from . import AuthorizeRequestTokenWithConsole
from launchpadlib.credentials import UnencryptedFileCredentialStore
from textwrap import dedent

logging.basicConfig(level=logging.DEBUG)
logger = logging.getLogger()
logger.name = 'hookshelper'

LP_APP = 'pbuilderjenkins-hookshelper'
LP_ENV = 'production'
CREDENTIAL_STORE_PATH = os.getenv('HOME') + '/.launchpad.credentials'
credential_store = UnencryptedFileCredentialStore(CREDENTIAL_STORE_PATH)
authorization_engine = AuthorizeRequestTokenWithConsole(LP_ENV, LP_APP)
launchpad = None
max_retries = 3

hook_template = dedent("""\
    #!/bin/sh
    set -e
    . /etc/lsb-release

    echo deb {ppa_url} $DISTRIB_CODENAME main >> /etc/apt/sources.list

    apt-key adv --keyserver keyserver.ubuntu.com --recv-keys {fingerprint}""")


def _get_launchpad():
    global launchpad
    if launchpad is None:
        launchpadlib_dir = '/tmp/.launchpadlib/{}'.format(os.getpid())
        try:
            launchpad = Launchpad.login_with(
                LP_APP, LP_ENV, credential_store=credential_store,
                authorization_engine=authorization_engine,
                launchpadlib_dir=launchpadlib_dir)
        except IOError as e:
            raise EnvironmentError('{}: {}'.format(e.strerror, e.filename))
    return launchpad


def get_ppa(person, ppa_name):
    """Get launchpad ppa object"""
    lp_exception = None
    retry_count = 0
    while retry_count < max_retries:
        try:
            launchpad = _get_launchpad()
            person = launchpad.people(person)
            return person.getPPAByName(name=ppa_name)
        except Exception as lp_exception:
            logging.error('Exception: {}'.format(lp_exception))
            retry_count += 1
    raise lp_exception


def get_ppa_fingerprint(person, ppa_name):
    """Get ppa fingerprint"""
    archive = get_ppa(person, ppa_name)
    return archive.signing_key_fingerprint


def parse_ppa_hook(hook_name):
    """Get hook_name in the form of D09add_ppa~$person~$ppaname and return
    (person, ppaname) pair or None in case it is not possible to parse the
    hook"""
    match = re.match('^D09add_ppa~([^~]+)~([^~]+)$', hook_name)
    if match:
        return (match.group(1), match.group(2))
    return None


def create_ppa_hook(hook_name, parsed_hook, target_directory):
    """Create an add_ppa hook in target_directory for hooks in the form of
    D09add_ppa~$person~$ppaname"""

    person = parsed_hook[0]
    ppa_name = parsed_hook[1]

    archive = get_ppa(person, ppa_name)
    if archive.private:
        launchpad = _get_launchpad()
        me = launchpad.people[launchpad.me.name]
        ppa_url = me.getArchiveSubscriptionURL(archive=archive)
    else:
        ppa_url = "http://ppa.launchpad.net/{person}/{ppa_name}/ubuntu"
        ppa_url = ppa_url.format(person=person, ppa_name=ppa_name)

    fingerprint = get_ppa_fingerprint(person, ppa_name)
    hook = hook_template.format(ppa_url=ppa_url, fingerprint=fingerprint)
    hook_name_path = os.path.join(target_directory, hook_name)

    hookfile = open(hook_name_path, 'w')
    hookfile.write(hook)
    hookfile.close()
    os.chmod(hook_name_path, 0775)
