# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.

# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.

__copyright__ = '2012 Canonical Ltd - ' + \
                'Sergio Schvezov <sergio.schvezov@canonical.com>'

import os
import subprocess
import logging
import shutil
import tempfile
from . import create_ppa_hook, parse_ppa_hook

from textwrap import dedent

logging.basicConfig(level=logging.DEBUG)
logger = logging.getLogger()
logger.name = 'pbuilderjenkins'
max_retries = 3


def bzr_build(recipe, target):
    """Executes the bzr build command which sets up from a recipe"""
    try:
        subprocess.check_call(['bzr', 'build', recipe, target])
    except subprocess.CalledProcessError as e:
        raise RuntimeError('Error while preparing from recipe: ' + e.message)


def create_recipe(main_branch, packaging_branch=None, target_branch=None):
    """Creates a recipe

    main_branch from launchpad
    packaging_branch specify from launchpad if packaging
    is on diferent branch
    target_branch is the launchpad branch to merge with

    """
    recipe = dedent('''\
                    # bzr-builder format 0.4
                    {main_branch}
                    {merge_cmd}
                    {nest_part_cmd}
                    ''')

    if target_branch != None:
        merge_cmd = 'merge target {}'.format(target_branch)
    else:
        merge_cmd = '# No target branch'
    if packaging_branch != None:
        nest_part_cmd = 'nest-part ubuntu {} debian'\
                    .format(packaging_branch)
    else:
        nest_part_cmd = '# No packaging branch'
    return recipe.format(main_branch=main_branch,
                         merge_cmd=merge_cmd,
                         nest_part_cmd=nest_part_cmd)


def hook_provision(hooks_repo_dir, hook_wishlist, hooks_dir):
    """Copies hooks to the location where hooks are executed from."""
    if not hooks_repo_dir:
        return

    if not os.path.isdir(hooks_repo_dir):
        logger.warning('Hook directory missing: ' + hooks_repo_dir)
        return

    hook_list = os.listdir(hooks_repo_dir)
    for hook in hook_wishlist:
        parsed_hook = parse_ppa_hook(hook)
        if parsed_hook is not None:
            logger.debug('Creating hook from {}'.format(hook))
            logger.debug('Parsed into {0}, {1}'.format(*parsed_hook))
            create_ppa_hook(hook, parsed_hook, hooks_dir)
        elif hook in hook_list:
            logger.debug('Adding hook: {}'.format(hook))
            shutil.copy(hooks_repo_dir + os.sep + hook, hooks_dir)


def hook_validate(hook_wishlist, hooks_dir):
    """Validates that all desired hooks are provisioned."""
    hook_list = os.listdir(hooks_dir)
    for hook in hook_wishlist:
        if hook not in hook_list:
            raise EnvironmentError('{} not found in hooks'.format(hook))


def invoke_hooks(hooks_dir, stage):
    """Invokes hooks

    hooks_dir is the location to search for hooks to execute
    stage is the prepended filename filter

    """
    if not hooks_dir:
        logger.warning('No hooks setup to run, skipping')
        return
    logger.info('Running hooks')
    try:
        for hook in sorted(os.listdir(hooks_dir)):
            if hook[0] is stage:
                logger.debug('Running hook ' + hook)
                subprocess.check_call([hooks_dir + os.sep + hook, ])
    except subprocess.CalledProcessError:
        raise RuntimeError('Could not execute hook: ' + hook)


def clean_hooks(hooks_dir):
    """Deletes files from hook directory

    hooks_dir is the location to delete from

    """
    if not hooks_dir:
        return
    logger.info('Deleting hook directory: ' + hooks_dir)
    shutil.rmtree(hooks_dir)


class PBuilder(object):
    """Intended to be inherited from to create pbuilderjenkins workers"""

    hooks_repo_dir = '/usr/share/pbuilderjenkins/hooks'

    def __init__(self,
                 main_branch,
                 packaging_branch=None,
                 target_branch=None,
                 extra_hooks_dir=None,
                 work_dir='work',
                 parallel_jobs=1):
        """Creates a Builder object

            main_branch from launchpad
            packaging_branch specify from launchpad if packaging
                is on diferent branch
            target_branch is the launchpad branch to merge with
            extra_hooks_dir if you have additional hooks
            work_dir defaults to 'work'
        """

        self._main_branch = main_branch
        self._packaging_branch = packaging_branch
        self._target_branch = target_branch
        self._work_dir = work_dir
        self._extra_hooks_dir = extra_hooks_dir
        self._parallel_jobs = parallel_jobs


    def hooks_setup(self, hook_wishlist):
        """Creates hook directory to provision specified hooks"""

        logger.info('Setting up hooks')

        if not hook_wishlist:
            logger.warning('Empty hook list')
            hook_wishlist = []

        if isinstance(hook_wishlist, basestring):
            hook_wishlist = [hook_wishlist, ]

        #apt-get update is always added and implicitely executed
        logger.info('Adding D09apt_update hook ("apt-get update" call)')
        hook_wishlist.append('D09apt_update')
        hooks_dir = tempfile.mkdtemp()
        logger.debug('Temporary hooks directory: ' + hooks_dir)
        hook_provision(self.hooks_repo_dir, hook_wishlist, hooks_dir)
        hook_provision(self._extra_hooks_dir,
                       hook_wishlist,
                       hooks_dir)
        hook_validate(hook_wishlist, hooks_dir)
        return hooks_dir

    def execute_build(self):
        """Intended to be implemented, if not this makes sure
        an exception is raised.
        """
        raise NotImplementedError('Base class has no implementation')

    def _branch_setup(self):
        """Sets up the required branches using recipes."""
        logging.info('Branch setup taking place using recipes')
        recipe = create_recipe(main_branch=self._main_branch,
                               packaging_branch=self._packaging_branch,
                               target_branch=self._target_branch)
        recipe_path = 'trunk.recipe'
        with open(recipe_path, 'w') as recipe_file:
            recipe_file.write(recipe)
        bzr_exception = None
        retry_count = 0
        while retry_count < max_retries:
            try:
                bzr_build(recipe_path, self._work_dir)
                return True
            except Exception as bzr_exception:
                logging.error('Exception: {}'.format(bzr_exception))
                retry_count += 1
        raise bzr_exception

    def _workdir_setup(self):
        """Creates the work directory to build in."""
        if os.path.isdir(self._work_dir) is True:
            raise EnvironmentError(
                'work directory {} exists'.format(self._work_dir))
        try:
            os.mkdir(self._work_dir)
        except OSError:
            raise EnvironmentError(
                'Cannot create work directory ' + self._work_dir)


class LocalPBuilder(PBuilder):
    """Local builds implementation of PBuilder."""

    def _build(self):
        """Builds in local environment without a chroot."""
        logger.info('Local build starting...')
        try:
            subprocess.check_call(['sudo',
                                   'mk-build-deps',
                                   '--install',
                                   '--tool',
                                   'apt-get -y --force-yes',
                                   '--build-dep',
                                   'debian/control'])
            subprocess.check_call(['debuild',
                                   '-uc',
                                   '-us',
                                   '-d'])
        except subprocess.CalledProcessError:
            raise RuntimeError('Local build failed')

    def execute_build(self, hooks=None):
        """Executes the build."""
        self._workdir_setup()
        self._branch_setup()

        hooks_dir = self.hooks_setup(hooks)
        os.chdir(self._work_dir)

        invoke_hooks(hooks_dir, 'H')
        invoke_hooks(hooks_dir, 'D')
        self._build()
        invoke_hooks(hooks_dir, 'B')
        clean_hooks(hooks_dir)


class ChrootPBuilder(PBuilder):
    """Chroot builds implementation of PBuilder."""

    reports = 'ReportDir'
    output = 'output'

    def __init__(self,
                 main_branch,
                 packaging_branch=None,
                 target_branch=None,
                 extra_hooks_dir=None,
                 work_dir='work',
                 result_dir='results',
                 basetgz='base',
                 parallel_jobs=1
             ):
        """
        Creates a Builder object

        main_branch from launchpad
        packaging_branch specify from launchpad if packaging
          is on diferent branch
        target_branch is the launchpad branch to merge with
        extra_hooks_dir if you have additional hooks
        work_dir defaults to 'work'
        result_dir result directory to use defaults to 'results'
        basetgz is the base chroot to use for building
          defaults to 'base'
        """
        super(ChrootPBuilder, self).__init__(main_branch,
                                             packaging_branch,
                                             target_branch,
                                             extra_hooks_dir,
                                             work_dir,
                                             parallel_jobs)
        self._base_path = os.path.abspath(os.getcwd())
        self._result_dir = os.path.join(self._base_path, work_dir, result_dir)
        self._basetgz = '/var/cache/pbuilder/' + basetgz + '.tgz'
        self._input_file = os.path.join(self._base_path, self.reports)
        self._output_dir = os.path.join(self._base_path, work_dir,
                                        self.output)

    def _build(self, hook_dir):
        '''Builds with pdebuild environment in a chroot'''
        if os.path.exists(self._basetgz) is not True:
            raise EnvironmentError(
                'Specified chroot environment ' \
                '{} does not exist'.format(self._basetgz))
        logger.info('Chroot build starting...')
        pdebuild_options = [
                   '--',
                   '--inputfile',
                   self._input_file,
                   '--bindmounts',
                   self._result_dir,
                   '--basetgz',
                   self._basetgz,
                   '--buildresult',
                   self._output_dir,
                   ]
        logger.info('pdebuild_options: {}'.format(pdebuild_options))
        if hook_dir:
            pdebuild_options += ['--hookdir', hook_dir]
        command = ['bzr',
                   'bd',
                   '--split',
                   '--orig-dir',
                   os.path.join(self._base_path, 'orig-dir'),
                   '--build-dir',
                   os.path.join(self._base_path, 'build-dir'),
                   '--builder=pdebuild',
                   '--',
                   "'{}'".format(' '.join(pdebuild_options)),
                   ]
        command = ' '.join(command)
        logger.info('command: %s' % (command))
        try:
            os.environ['DEB_BUILD_OPTIONS'] = "parallel={}".format(
                self._parallel_jobs)
            subprocess.check_call(command, shell=True)
        except subprocess.CalledProcessError:
            raise RuntimeError('pdebuild failed')

    def _workdir_setup(self):
        """Chroot building sets up additional directories
        for postprocessing.
        """
        super(ChrootPBuilder, self)._workdir_setup()

        try:
            os.mkdir(self._result_dir)
            with open(self._input_file, 'w') as f:
                f.write('RESULT_DIR=' + self._result_dir + '\n')
            os.mkdir(self._output_dir)
            # Output directory needs to be erasable later as the
            # executing user
            os.chmod(self._output_dir, 07775)
        except OSError:
            raise EnvironmentError(
                'Ancilliary directory creation for chroot build failed')

    def execute_build(self, hooks=None):
        """Executes the build."""
        self._workdir_setup()
        self._branch_setup()

        hooks_dir = self.hooks_setup(hooks)
        os.chdir(self._work_dir)

        # This is very hacky, but used to do any cleanups before building
        invoke_hooks(hooks_dir, 'H')
        self._build(hooks_dir)
        clean_hooks(hooks_dir)
