# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.

# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.

__copyright__ = '2012 Canonical Ltd - ' + \
                'Sergio Schvezov <sergio.schvezov@canonical.com>'

from mock import MagicMock
from mock import patch
from testtools import TestCase
from testtools import skip
from testtools.matchers import raises

from pbuilderjenkins.pbuilderjenkins import PBuilder


class BaseBuilderTest(TestCase):

    def setUp(self):
        super(BaseBuilderTest, self).setUp()
        self.main_branch = 'lp:main'
        self.packaging_branch = 'lp:packaging'
        self.target_branch = 'lp:merge'

    def test_exception_on_build(self):
        """
        Calling build on a PBuilder instance MUST
        raise a NotImplementedError.
        """
        exception_message = 'Base class has no implementation'
        builder = PBuilder(self.main_branch)
        self.assertThat(builder.execute_build,
                        raises(NotImplementedError(exception_message)))

    def test_builder_with_no_main_and_with_target_branch(self):
        """
        Creating a PBuilder instance with no main branch MUST raise a TypeError.
        """

        self.assertThat(lambda: PBuilder(target_branch=self.target_branch),
                        raises(TypeError))

    @patch('__builtin__.open', create=True)
    @patch('pbuilderjenkins.pbuilderjenkins.create_recipe', create=True)
    @patch('pbuilderjenkins.pbuilderjenkins.bzr_build', create=True)
    def test_branch_setup_with_only_main_branch(self,
                                                bzr_mock,
                                                recipe_mock,
                                                open_mock):
        """
        Invoking _branch_setup from a PBuilder instance with only
        the main branch MUST request to create a recipe and
        provision the recipe on disk.
        """
        open_mock.return_value = MagicMock(spec=file)
        # Given
        builder = PBuilder(self.main_branch)
        # When
        result = builder._branch_setup()
        # Then
        self.assertTrue(result)
        bzr_mock.assert_called_once_with('trunk.recipe', 'work')
        open_mock.assert_called_once_with('trunk.recipe', 'w')
        recipe_mock.assert_called_once_with(main_branch=self.main_branch,
                                            packaging_branch=None,
                                            target_branch=None)

    @patch('__builtin__.open', create=True)
    @patch('pbuilderjenkins.pbuilderjenkins.create_recipe', create=True)
    @patch('pbuilderjenkins.pbuilderjenkins.bzr_build', create=True)
    def test_branch_setup_with_main_and_packaging_branch(self,
                                                         bzr_mock,
                                                         recipe_mock,
                                                         open_mock):
        """
        Invoking _branch_setup from a PBuilder instance with
        the main and packaging branch MUST request to create a
        recipe and provision the recipe on disk.
        """
        open_mock.return_value = MagicMock(spec=file)
        # Given
        builder = PBuilder(self.main_branch, self.packaging_branch)
        # When
        result = builder._branch_setup()
        # Then
        self.assertTrue(result)
        bzr_mock.assert_called_once_with('trunk.recipe', 'work')
        open_mock.assert_called_once_with('trunk.recipe', 'w')
        recipe_mock.assert_called_once_with(main_branch=self.main_branch,
                                            packaging_branch=self.packaging_branch,
                                            target_branch=None)

    @patch('__builtin__.open', create=True)
    @patch('pbuilderjenkins.pbuilderjenkins.create_recipe', create=True)
    @patch('pbuilderjenkins.pbuilderjenkins.bzr_build', create=True)
    def test_branch_setup_with_main_packaging_and_target_branch(self,
                                                                bzr_mock,
                                                                recipe_mock,
                                                                open_mock):
        """
        Invoking _branch_setup from a PBuilder instance with
        the main, packaging and target branch MUST request
        to create a recipe and provision the recipe on disk.
        """
        open_mock.return_value = MagicMock(spec=file)
        # Given
        builder = PBuilder(self.main_branch,
                           self.packaging_branch,
                           self.target_branch)
        # When
        result = builder._branch_setup()
        # Then
        self.assertTrue(result)
        bzr_mock.assert_called_once_with('trunk.recipe', 'work')
        open_mock.assert_called_once_with('trunk.recipe', 'w')
        recipe_mock.assert_called_once_with(main_branch=self.main_branch,
                                            packaging_branch=self.packaging_branch,
                                            target_branch=self.target_branch)

    @patch('__builtin__.open', create=True)
    @patch('pbuilderjenkins.pbuilderjenkins.create_recipe', create=True)
    @patch('pbuilderjenkins.pbuilderjenkins.bzr_build', create=True)
    def test_branch_setup_with_main_and_target_branch(self,
                                                      bzr_mock,
                                                      recipe_mock,
                                                      open_mock):
        """
        Invoking _branch_setup from a PBuilder instance with
        the main  and target branch MUST request
        to create a recipe and provision the recipe on disk.
        """
        open_mock.return_value = MagicMock(spec=file)
        # Given
        builder = PBuilder(self.main_branch,
                           target_branch=self.target_branch)
        # When
        result = builder._branch_setup()
        # Then
        self.assertTrue(result)
        bzr_mock.assert_called_once_with('trunk.recipe', 'work')
        open_mock.assert_called_once_with('trunk.recipe', 'w')
        recipe_mock.assert_called_once_with(main_branch=self.main_branch,
                                            packaging_branch=None,
                                            target_branch=self.target_branch)

    @patch('__builtin__.open', create=True)
    @patch('pbuilderjenkins.pbuilderjenkins.create_recipe', create=True)
    @patch('pbuilderjenkins.pbuilderjenkins.bzr_build', create=True,
           side_effect=Exception("mock"))
    def test_branch_setup_with_bzr_exception(self,
                                             bzr_mock,
                                             recipe_mock,
                                             open_mock):
        """
        Invoking _branch_setup and simulate a recurring exception, the
        call should return False (no infinite loop).
        """
        open_mock.return_value = MagicMock(spec=file)
        # Given
        builder = PBuilder(self.main_branch)
        # When / Then
        self.assertRaises(Exception, builder._branch_setup)

    @patch('__builtin__.open', create=True)
    @patch('pbuilderjenkins.pbuilderjenkins.create_recipe', create=True)
    @patch('pbuilderjenkins.pbuilderjenkins.bzr_build', create=True,
           side_effect=[Exception("mock1"), None])
    def test_branch_setup_with_single_bzr_exception(self,
                                                    bzr_mock,
                                                    recipe_mock,
                                                    open_mock):
        """
        Invoking _branch_setup and simulate a single exception, the
        call should return True (the retry worked).
        """
        open_mock.return_value = MagicMock(spec=file)
        # Given
        builder = PBuilder(self.main_branch)
        # When
        result = builder._branch_setup()
        # Then
        self.assertTrue(result)


    @patch('os.path.isdir', create=True)
    @patch('os.mkdir', create=True)
    def test_work_dir_setup_defaults(self, mkdir_mock, isdir_mock):
        """
        Invoking _workdir_setup from a PBuilder instance that was NOT
        initialized with a specific 'work_dir' argument MUST
        create the default 'work' directory.
        """
        # Given
        builder = PBuilder(self.main_branch)
        # When
        builder._workdir_setup()
        # Then
        mkdir_mock.assert_called_once_with('work')

    @patch('os.path.isdir', create=True)
    @patch('os.mkdir', create=True)
    def test_work_dir_setup_configured(self, mkdir_mock, isdir_mock):
        """
        Invoking _workdir_setup from a PBuilder instance that was
        initialized with a specific 'work_dir' argument MUST
        create the initialized 'work' directory.
        """
        # Given
        builder = PBuilder(self.main_branch, work_dir='another')
        # When
        builder._workdir_setup()
        # Then
        mkdir_mock.assert_called_once_with('another')

    @skip('Unfinished')
    @patch('os.path.isdir', create=True)
    @patch('os.mkdir', create=True)
    def test_work_dir_setup_fails_with_existing_dir(self, mkdir_mock, isdir_mock):
        """
        Invoking _workdir_setup from a PBuilder instance MUST
        raise an OSError when the 'work' directory already exists.
        """
        # Given
        builder = PBuilder(self.main_branch)
        # When
        builder._workdir_setup()
        # Then
        mkdir_mock.assert_called_once_with('another')
