# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.

# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.

__copyright__ = '2013 Canonical Ltd - ' + \
                'Martin Mrazik <martin.mrazik@canonical.com>'

from testtools import TestCase
from mock import patch
from mock import MagicMock
from mock import call
from testtools.matchers import Equals
from testscenarios import TestWithScenarios
import os

from textwrap import dedent
from pbuilderjenkins.hookshelper import get_ppa
from pbuilderjenkins.hookshelper import get_ppa_fingerprint
from pbuilderjenkins.hookshelper import parse_ppa_hook
from pbuilderjenkins.hookshelper import create_ppa_hook


class GetPPATest(TestCase):
    def test_get_get_ppa(self):
        """ Test that meaningful data is returned"""
        archive = "mock archive"
        launchpad_mock = MagicMock()
        person_mock = MagicMock()
        launchpad_mock.people = lambda x: person_mock
        person_mock.getPPAByName = lambda name: archive
        with patch('pbuilderjenkins.hookshelper.launchpad', launchpad_mock):
            result = get_ppa('person', 'ppa_name')
            self.assertThat(result, Equals(archive)) 

    def test_get_get_ppa_exception(self):
        """ Test that an exception does not loop forever."""
        launchpad_mock = MagicMock()
        launchpad_mock.people = MagicMock(side_effect=Exception("mock"))
        with patch('pbuilderjenkins.hookshelper.launchpad', launchpad_mock):
            self.assertRaises(Exception, get_ppa, 'person', 'ppa_name')

    def test_get_get_ppa_single_exception(self):
        """ Test that a single exception returns meaningful data."""
        archive = "mock archive"
        launchpad_mock = MagicMock()
        person_mock = MagicMock()
        launchpad_mock.people = lambda x: person_mock
        person_mock.getPPAByName = MagicMock(side_effect=[Exception("mock"),
                                                          archive])
        with patch('pbuilderjenkins.hookshelper.launchpad', launchpad_mock):
            result = get_ppa('person', 'ppa_name')
            self.assertThat(result, Equals(archive)) 


class GetPPAFingerPrintTest(TestCase):
    def test_get_ppa_fingerprint(self):
        """ Test that get_ppa_fingerprint returns a meaninfgul value"""
        fingerprint = 'fingerprint'
        launchpad_mock = MagicMock()
        person_mock = MagicMock()
        archive_mock = MagicMock()
        archive_mock.signing_key_fingerprint = fingerprint
        launchpad_mock.people = lambda x: person_mock
        person_mock.getPPAByName = lambda name: archive_mock
        with patch('pbuilderjenkins.hookshelper.launchpad', launchpad_mock):
            result = get_ppa_fingerprint('person', 'ppa_name')
            self.assertThat(result, Equals(fingerprint))


class ParsePPAHookTest(TestWithScenarios, TestCase):
    scenarios = [
        ('H09disable_doxygen',
         {'hook_name': '',
          'team_ppa_pair': None}),
        ('D09add_ppa-private-ps-quality-team-testing',
         {'hook_name': '',
          'team_ppa_pair': None}),
        ('D09add_ppa-phablet-team-ppa',
         {'hook_name': '',
          'team_ppa_pair': None}),
        ('A10checklicenseheaders',
         {'hook_name': '',
          'team_ppa_pair': None}),
        ('empty',
         {'hook_name': '',
          'team_ppa_pair': None}),
        ('D09add_ppa~mir-team~staging',
         {'hook_name': 'D09add_ppa~mir-team~staging',
          'team_ppa_pair': ('mir-team', 'staging'),
          'fingerprint': 'AX234FDD',
          'ppa_url': 'http://ppa.launchpad.net/mir-team/staging/ubuntu',
          'private_archive': False,
          }),
        ('D09add_ppa~mir-team',
         {'hook_name': '',
          'team_ppa_pair': None}),
        ('D09add_ppa~qt5~beta1',
         {'hook_name': 'D09add_ppa~qt5~beta1',
          'private_archive': False,
          'team_ppa_pair': ('qt5', 'beta1'),
          'ppa_url': 'http://ppa.launchpad.net/qt5/beta1/ubuntu',
          'fingerprint': 'B132DAS'}),
        ('D09add_ppa~qt5~beta1',
         {'hook_name': 'D09add_ppa~qt5~beta1',
          'private_archive': True,
          'team_ppa_pair': ('qt5', 'beta1'),
          'ppa_url': 'https://user:LSDFASD@private-ppa.launchpad.net' +
                     '/qt5/beta1/ubuntu',
          'fingerprint': 'B132DAS'}),
    ]

    def test_parse_ppa_hook(self):
        """ Test parse_ppa_hook with different hook_names"""
        # Given/When
        result = parse_ppa_hook(self.hook_name)
        # Then
        self.assertThat(result, Equals(self.team_ppa_pair))

    def _get_hook(self, ppa_url, fingerprint):
        hook_template = dedent("""\
        #!/bin/sh
        set -e
        . /etc/lsb-release

        echo deb {ppa_url} $DISTRIB_CODENAME main >> /etc/apt/sources.list

        apt-key adv --keyserver keyserver.ubuntu.com --recv-keys {fingerprint}""")
        return hook_template.format(ppa_url=ppa_url, fingerprint=fingerprint)

    @patch('os.chmod')
    def test_create_ppa_hook(self, chmod_mock):
        """ Test create_ppa_hook with different hook names """
        # Given
        target_dir = '/tmp/some_dir'
        file_mock = MagicMock()
        open_mock = MagicMock(return_value=file_mock)
        archive_mock = MagicMock()
        launchpad = MagicMock()
        launchpad.me.name = 'name'
        me = MagicMock()
        me.getArchiveSubscriptionURL = lambda archive: self.ppa_url
        launchpad.people = {}
        launchpad.people['name'] = me
        if hasattr(self, 'private_archive'):
            archive_mock.private = self.private_archive
        get_ppa = lambda x, y: archive_mock
        with patch("__builtin__.open", open_mock), \
                patch("pbuilderjenkins.hookshelper.get_ppa_fingerprint",
                      lambda x, y: self.fingerprint), \
                patch("pbuilderjenkins.hookshelper.get_ppa", get_ppa), \
                patch("pbuilderjenkins.hookshelper.launchpad", launchpad):
        # When
            if self.team_ppa_pair:
                create_ppa_hook(self.hook_name, self.team_ppa_pair, target_dir)

        # Then
        if self.team_ppa_pair is None:
            self.assertThat(open_mock.called, Equals(False))
            self.assertThat(file_mock.write.called, Equals(False))
        else:
            hook_filename = os.path.join(target_dir, self.hook_name)
            expected_calls = [call(hook_filename, 'w')]
            self.assertThat(open_mock.call_args_list, Equals(expected_calls))
            expected_hook = self._get_hook(self.ppa_url, self.fingerprint)
            self.assertThat(file_mock.write.call_args_list,
                            Equals([call(expected_hook)]))
            self.assertThat(chmod_mock.call_args_list,
                            Equals([call(hook_filename, 0775)]))
