# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.

# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.

""" These tests are actually not testing any python code but a bash code
from the hooks directory. In particular the
add_autotools_dh_auto_configure_option and
add_cmake_dh_auto_configure_option functions.

I'm just using python to execute the tests and gather results easily in some
standard (i.e. xml) format.
"""

__copyright__ = '2012 Canonical Ltd - ' + \
                 'Martin Mrazik <martin.mrazik@canonical.com>'

from testtools import TestCase
from textwrap import dedent
import tempfile
import shutil
import os
import subprocess


class InvokeAddAutoConfigureOptionTest(TestCase):
    add_function = None

    def run_add_dh_auto_configure_option(self, rules, option):
        tmpdir = tempfile.mkdtemp()
        os.mkdir(tmpdir + '/debian')
        filename = tmpdir + '/debian/rules'
        open(filename, 'w').write(rules)
        shell_code = '. hooks/%s; cd %s; %s %s' % (self.add_function,
                                                   tmpdir,
                                                   self.add_function,
                                                   option)
        cmake_call = ['bash', '-c', shell_code]
        subprocess.check_call(cmake_call)
        result = open(filename).read()
        shutil.rmtree(tmpdir)
        return result


class TestAutoToolsAddConfigureOption(InvokeAddAutoConfigureOptionTest):
    add_function = 'add_autotools_dh_auto_configure_option'

    def test_no_override(self):
        rules = dedent("""\
             #!/usr/bin/make -f

             %:
             \tdh $@
        """)
        expected = dedent("""\
             #!/usr/bin/make -f

             %:
             \tdh $@

             override_dh_auto_configure:
             \tdh_auto_configure -- --gcov

             """)
        result = self.run_add_dh_auto_configure_option(rules, '--gcov')
        self.assertEqual(expected, result)

    def test_unity_chromium_extension_add_gcov(self):
        """Tests the add function in case override_dh_auto_configure.
           This particular example is from unity-chromium-extension
           packaging"""

        rules = dedent("""\
            #!/usr/bin/make -f
            # -*- makefile -*-
            # Uncomment this to turn on verbose mode.
            #export DH_VERBOSE=1

            override_dh_autoreconf:
                NOCONFIGURE=1 dh_autoreconf ./autogen.sh

            override_dh_auto_configure:
                cp debian/unity-webapps.pem .
                dh_auto_configure -- --enable-tests=yes

        """)
        expected = dedent("""\
            #!/usr/bin/make -f
            # -*- makefile -*-
            # Uncomment this to turn on verbose mode.
            #export DH_VERBOSE=1

            override_dh_autoreconf:
                NOCONFIGURE=1 dh_autoreconf ./autogen.sh

            override_dh_auto_configure:
                cp debian/unity-webapps.pem .
                dh_auto_configure -- --gcov --enable-tests=yes

        """)
        result = self.run_add_dh_auto_configure_option(rules, '--gcov')
        self.assertEqual(expected, result)

    def test_libpam_icaclient_add_gcov(self):
        """Tests the add function in case override_dh_auto_configure exists
           but dh_auto_configure is not called with any parames there.
           This particular example is from libpam-icaclient
           packaging"""

        rules = dedent("""\
            #!/usr/bin/make -f

            %:
                dh $@

            override_dh_auto_configure:
                ./autogen.sh
                dh_auto_configure

            override_dh_install:
                dh_install
        """)
        expected = dedent("""\
            #!/usr/bin/make -f

            %:
                dh $@

            override_dh_auto_configure:
                ./autogen.sh
                dh_auto_configure -- --gcov

            override_dh_install:
                dh_install
        """)
        result = self.run_add_dh_auto_configure_option(rules, '--gcov')
        self.assertEqual(expected, result)


class TestCmakeAddConfigureOption(InvokeAddAutoConfigureOptionTest):
    add_function = 'add_cmake_dh_auto_configure_option'

    def test_no_override(self):
        rules = dedent("""\
             #!/usr/bin/make -f

             %:
             \tdh $@
        """)
        expected = dedent("""\
             #!/usr/bin/make -f

             %:
             \tdh $@

             override_dh_auto_configure:
             \tdh_auto_configure -- -DCMAKE_BUILD_TYPE=coverage

             """)
        result = self.run_add_dh_auto_configure_option(rules, '-DCMAKE_BUILD_TYPE=coverage')
        self.assertEqual(expected, result)

    def test_compiz_packaging(self):
        rules = dedent("""\
             #!/usr/bin/make -f

             override_dh_auto_configure:
                 # currently, segfault if CMAKE_BUILD_TYPE=Release
                 LDFLAGS=$(shell echo $$LDFLAGS | sed -e 's/\-Wl,-Bsymbolic-functions //')
             ifeq ($(DEB_HOST_ARCH),$(findstring $(DEB_HOST_ARCH), $(gles2_architectures)))
                 dh_auto_configure -- -DCOMPIZ_BUILD_WITH_RPATH=FALSE -DCOMPIZ_DEFAULT_PLUGINS=\\"$(DEFAULT_PLUGINS)\\" -DCMAKE_BUILD_TYPE=RelWithDebInfo -DCOMPIZ_PACKAGING_ENABLED=TRUE -DUSE_GSETTINGS=ON -DCOMPIZ_DISABLE_GS    _SCHEMAS_INSTALL=OFF -DCOMPIZ_DISABLE_PLUGIN_KDE=ON -DCOMPIZ_BUILD_TESTING=OFF -DUSE_KDE4=OFF -DUSE_GCONF=OFF -DBUILD_GLES=ON
             else
                 dh_auto_configure -- -DCOMPIZ_BUILD_WITH_RPATH=FALSE -DCOMPIZ_DEFAULT_PLUGINS=\\"$(DEFAULT_PLUGINS)\\" -DCMAKE_BUILD_TYPE=RelWithDebInfo -DCOMPIZ_PACKAGING_ENABLED=TRUE -DUSE_GSETTINGS=ON -DCOMPIZ_DISABLE_GS    _SCHEMAS_INSTALL=OFF -DCOMPIZ_DISABLE_PLUGIN_KDE=ON -DCOMPIZ_BUILD_TESTING=OFF -DUSE_KDE4=OFF -DUSE_GCONF=OFF
             endif

             override_dh_install:
                 # cmake findcompiz_install use COMPIZ_DESTDIR and using COMPIZ_DESTDIR and
                 # DESTDIR together is completely broken upstream
                 # So, doing the same in debian/rules to find a way to install things in the

        """)
        expected = dedent("""\
             #!/usr/bin/make -f

             override_dh_auto_configure:
                 # currently, segfault if CMAKE_BUILD_TYPE=Release
                 LDFLAGS=$(shell echo $$LDFLAGS | sed -e 's/\-Wl,-Bsymbolic-functions //')
             ifeq ($(DEB_HOST_ARCH),$(findstring $(DEB_HOST_ARCH), $(gles2_architectures)))
                 dh_auto_configure -- -DCOMPIZ_BUILD_WITH_RPATH=FALSE -DCOMPIZ_DEFAULT_PLUGINS=\\"$(DEFAULT_PLUGINS)\\" -DCMAKE_BUILD_TYPE=coverage -DCOMPIZ_PACKAGING_ENABLED=TRUE -DUSE_GSETTINGS=ON -DCOMPIZ_DISABLE_GS    _SCHEMAS_INSTALL=OFF -DCOMPIZ_DISABLE_PLUGIN_KDE=ON -DCOMPIZ_BUILD_TESTING=OFF -DUSE_KDE4=OFF -DUSE_GCONF=OFF -DBUILD_GLES=ON
             else
                 dh_auto_configure -- -DCOMPIZ_BUILD_WITH_RPATH=FALSE -DCOMPIZ_DEFAULT_PLUGINS=\\"$(DEFAULT_PLUGINS)\\" -DCMAKE_BUILD_TYPE=coverage -DCOMPIZ_PACKAGING_ENABLED=TRUE -DUSE_GSETTINGS=ON -DCOMPIZ_DISABLE_GS    _SCHEMAS_INSTALL=OFF -DCOMPIZ_DISABLE_PLUGIN_KDE=ON -DCOMPIZ_BUILD_TESTING=OFF -DUSE_KDE4=OFF -DUSE_GCONF=OFF
             endif

             override_dh_install:
                 # cmake findcompiz_install use COMPIZ_DESTDIR and using COMPIZ_DESTDIR and
                 # DESTDIR together is completely broken upstream
                 # So, doing the same in debian/rules to find a way to install things in the

             """)
        result = self.run_add_dh_auto_configure_option(rules, '-DCMAKE_BUILD_TYPE=coverage')
        self.assertEqual(expected, result)

    def test_multiline_packaging(self):
        rules = dedent("""\
             #!/usr/bin/make -f

             override_dh_auto_configure:
                dh_auto_configure -- \
                    -DMY_OPTION=yes \
                    -DMY_SECOND_OPTION=NO

        """)
        expected = dedent("""\
             #!/usr/bin/make -f

             override_dh_auto_configure:
                dh_auto_configure -- -DCMAKE_BUILD_TYPE=coverage \
                    -DMY_OPTION=yes \
                    -DMY_SECOND_OPTION=NO

             """)
        result = self.run_add_dh_auto_configure_option(rules, '-DCMAKE_BUILD_TYPE=coverage')
        self.assertEqual(expected, result)

