# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.

# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.

__copyright__ = '2012 Canonical Ltd - ' + \
                'Sergio Schvezov <sergio.schvezov@canonical.com>'

from testtools import TestCase
from mock import patch
from mock import call
from testtools.matchers import Equals
from testtools.matchers import raises

import subprocess

from pbuilderjenkins.pbuilderjenkins import hook_validate
from pbuilderjenkins.pbuilderjenkins import invoke_hooks
from pbuilderjenkins.pbuilderjenkins import hook_provision


class InvokeHookTest(TestCase):

    @patch('os.listdir')
    @patch('subprocess.check_call')
    def test_raise_exception_when_error_calling_hooks(self,
                                                      check_call_mock,
                                                      listdir_mock):
        """
        When invoking hooks a RuntimeError MUST
        be raised if one fails to execute.
        """
        # Given
        listdir_mock.return_value = ['A00hook']
        check_call_mock.side_effect = \
            subprocess.CalledProcessError(1, 'hook')
        # When/Then
        self.assertThat(lambda: invoke_hooks('/dir', 'A'),
                        raises(RuntimeError))

    @patch('subprocess.check_call')
    def test_if_no_hooks_do_not_try_execute(self, check_call_mock):
        """
        When invoking hooks on no hook directory check_call MUST NOT
        be executed.
        """
        # Given
        hook_dir = None
        # When
        invoke_hooks(hook_dir, 'A')
        # Then
        self.assertThat(check_call_mock.called, Equals(False))

    @patch('os.listdir')
    @patch('subprocess.check_call')
    def test_calling_hooks_in_sorted_order(self,
                                           check_call_mock,
                                           listdir_mock):
        """
        Hooks must be invoked in alphanumeric sorted order.
        """
        # Given
        hook_wishlist = ['A10another_hook', 'A00hook', 'A05hooky']
        listdir_mock.return_value = hook_wishlist
        # When
        invoke_hooks('/dir', 'A')
        # Then
        expected = []
        hook_wishlist.sort()
        for hook in hook_wishlist:
            expected.append(call(['/dir/' + hook]))
        self.assertThat(check_call_mock.call_args_list, Equals(expected))


class ValidateHookTest(TestCase):

    @patch('os.listdir')
    def test_unfound_hook_shall_raise_exception(self, list_dir_mock):
        """
        Not finding a desired hook in the hooks directory MUST
        raise a EnvironmentError exception
        """
        # Given
        hook_wishlist = ['H00hook1', 'B00hook2', 'D11hook3']
        list_dir_mock.return_value = hook_wishlist[:-1]
        # When/Then
        self.assertThat(lambda: hook_validate(hook_wishlist, 'hook_dir'),
                        raises(EnvironmentError))
        self.assertThat(list_dir_mock.called, Equals(True))

    @patch('os.listdir')
    def test_find_all_desired_hooks(self, list_dir_mock):
        """
        Finding akk desired hooks in the hooks directory MUST
        raise a EnvironmentError exception
        """
        # Given
        hook_wishlist = ['H00hook1', 'B00hook2', 'D11hook3']
        list_dir_mock.return_value = hook_wishlist
        # When
        hook_validate(hook_wishlist, 'hook_dir')
        # Then
        self.assertThat(list_dir_mock.called, Equals(True))

    @patch('os.listdir')
    def test_find_all_desired_hooks_with_more_in_list(self, list_dir_mock):
        """
        Finding akk desired hooks in the hooks directory MUST
        raise a EnvironmentError exception
        """
        # Given
        hook_wishlist = ['H00hook1', 'B00hook2', 'D11hook3']
        list_dir_mock.return_value = hook_wishlist + ['A00more']
        # When
        hook_validate(hook_wishlist, 'hook_dir')
        # Then
        self.assertThat(list_dir_mock.called, Equals(True))


class ProvisionHookTest(TestCase):
    @patch('os.path.isdir')
    @patch('pbuilderjenkins.pbuilderjenkins.create_ppa_hook')
    @patch('os.listdir')
    def test_ppa_hook_generation(self, listdir_mock, create_ppa_hook_mock,
                                 isdir_mock):
        """
            Calling hook_provision with a hook in the form of
            "D09add_ppa~$person~$ppaname" MUST call create_ppa_hook
        """
        # Given
        hooks_repo_dir = '/repo'
        hook_wishlist = ['D09add_ppa~mir-team~staging',
                         'D09add_ppa-some-other-ppa']
        hooks_dir = '/tmp'
        # When
        hook_provision(hooks_repo_dir, hook_wishlist, hooks_dir)
        # Then
        self.assertThat(create_ppa_hook_mock.call_count, Equals(1))
